@@ -13,11 +13,11 @@
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 
 
-package Tk::Menubutton; 
+package Tk::Menubutton;
 require Tk;
 
-use vars qw($VERSION @ISA);
-$VERSION = '3.012'; # $Id: //depot/Tk8/Menubutton/Menubutton.pm#12$
+use vars qw($VERSION);
+$VERSION = '3.019'; # $Id: //depot/Tk8/Menubutton/Menubutton.pm#19$
 
 use base  qw(Tk::Widget);
 
@@ -102,15 +102,15 @@ sub InitObject
 sub ClassInit
 {
  my ($class,$mw) = @_;
- $mw->bind($class,"<FocusIn>",'NoOp');
- $mw->bind($class,"<Enter>",'Enter');
- $mw->bind($class,"<Leave>",'Leave');
- $mw->bind($class,"<1>",'ButtonDown');
- $mw->bind($class,"<Motion>",['Motion',"up",Ev('X'),Ev('Y')]);
- $mw->bind($class,"<B1-Motion>",['Motion',"down",Ev('X'),Ev('Y')]);
- $mw->bind($class,"<ButtonRelease-1>",'ButtonUp');
- $mw->bind($class,"<space>",'PostFirst');
- $mw->bind($class,"<Return>",'PostFirst');
+ $mw->bind($class,'<FocusIn>','NoOp');
+ $mw->bind($class,'<Enter>','Enter');
+ $mw->bind($class,'<Leave>','Leave');
+ $mw->bind($class,'<1>','ButtonDown');
+ $mw->bind($class,'<Motion>',['Motion','up',Ev('X'),Ev('Y')]);
+ $mw->bind($class,'<B1-Motion>',['Motion','down',Ev('X'),Ev('Y')]);
+ $mw->bind($class,'<ButtonRelease-1>','ButtonUp');
+ $mw->bind($class,'<space>','PostFirst');
+ $mw->bind($class,'<Return>','PostFirst');
  return $class;
 }
 
@@ -123,7 +123,7 @@ sub ButtonDown
 sub PostFirst
 {
  my $w = shift;
- my $menu = $w->cget("-menu");
+ my $menu = $w->cget('-menu');
  $w->Post();
  $menu->FirstEntry() if (defined $menu);
 }
@@ -142,9 +142,9 @@ sub Enter
  my $w = shift;
  $Tk::inMenubutton->Leave if (defined $Tk::inMenubutton);
  $Tk::inMenubutton = $w;
- if ($w->cget("-state") ne "disabled")
+ if ($w->cget('-state') ne 'disabled')
   {
-   $w->configure("-state","active")
+   $w->configure('-state','active')
   }
 }
 
@@ -153,9 +153,9 @@ sub Leave
  my $w = shift;
  $Tk::inMenubutton = undef;
  return unless Tk::Exists($w);
- if ($w->cget("-state") eq "active")
+ if ($w->cget('-state') eq 'active')
   {
-   $w->configure("-state","normal")
+   $w->configure('-state','normal')
   }
 }
 # Post --
@@ -174,9 +174,9 @@ sub Post
  my $w = shift;
  my $x = shift;
  my $y = shift;
- return if ($w->cget("-state") eq "disabled");
+ return if ($w->cget('-state') eq 'disabled');
  return if (defined $Tk::postedMb && $w == $Tk::postedMb);
- my $menu = $w->cget("-menu");
+ my $menu = $w->cget('-menu');
  return unless (defined($menu) && $menu->index('last') ne 'none');
 
  my $tearoff = $Tk::platform eq 'unix' || $menu->cget('-type') eq 'tearoff';
@@ -193,36 +193,36 @@ sub Post
   {
    Tk::Menu->Unpost(undef); # fixme
   }
- $Tk::cursor = $w->cget("-cursor");
- $Tk::relief = $w->cget("-relief");
- $w->configure("-cursor","arrow");
- $w->configure("-relief","raised");
+ $Tk::cursor = $w->cget('-cursor');
+ $Tk::relief = $w->cget('-relief');
+ $w->configure('-cursor','arrow');
+ $w->configure('-relief','raised');
  $Tk::postedMb = $w;
  $Tk::focus = $w->focusCurrent;
- $menu->activate("none"); 
+ $menu->activate('none');
  $menu->GenerateMenuSelect;
  # If this looks like an option menubutton then post the menu so
  # that the current entry is on top of the mouse. Otherwise post
  # the menu just below the menubutton, as for a pull-down.
 
- eval 
+ eval
   {local $SIG{'__DIE__'};
    my $dir = $w->cget('-direction');
    if ($dir eq 'above')
     {
-     $menu->post($w->rootx, $w->rooty - $menu->ReqHeight); 
+     $menu->post($w->rootx, $w->rooty - $menu->ReqHeight);
     }
    elsif ($dir eq 'below')
     {
-     $menu->post($w->rootx, $w->rooty + $w->Height); 
-    } 
+     $menu->post($w->rootx, $w->rooty + $w->Height);
+    }
    elsif ($dir eq 'left')
     {
      my $x = $w->rootx - $menu->ReqWidth;
      my $y = int((2*$w->rooty + $w->Height) / 2);
-     if ($w->cget("-indicatoron") == 1 && defined($w->cget("-textvariable")))
+     if ($w->cget('-indicatoron') == 1 && defined($w->cget('-textvariable')))
       {
-       $menu->PostOverPoint($x,$y,$menu->FindName($w->cget("-text")))
+       $menu->PostOverPoint($x,$y,$menu->FindName($w->cget('-text')))
       }
      else
       {
@@ -233,25 +233,25 @@ sub Post
     {
      my $x = $w->rootx + $w->Width;
      my $y = int((2*$w->rooty + $w->Height) / 2);
-     if ($w->cget("-indicatoron") == 1 && defined($w->cget("-textvariable")))
+     if ($w->cget('-indicatoron') == 1 && defined($w->cget('-textvariable')))
       {
-       $menu->PostOverPoint($x,$y,$menu->FindName($w->cget("-text")))
+       $menu->PostOverPoint($x,$y,$menu->FindName($w->cget('-text')))
       }
      else
       {
        $menu->post($x,$y);
       }
-    }  
+    }
    else
     {
-     if ($w->cget("-indicatoron") == 1 && defined($w->cget("-textvariable")))
+     if ($w->cget('-indicatoron') == 1 && defined($w->cget('-textvariable')))
       {
        if (!defined($y))
         {
          $x = $w->rootx+$w->width/2;
          $y = $w->rooty+$w->height/2
         }
-       $menu->PostOverPoint($x,$y,$menu->FindName($w->cget("-text")))
+       $menu->PostOverPoint($x,$y,$menu->FindName($w->cget('-text')))
       }
      else
       {
@@ -262,7 +262,7 @@ sub Post
  if ($@)
   {
    Tk::Menu->Unpost;
-   die $@ 
+   die $@
   }
 
  $Tk::tearoff = $tearoff;
@@ -301,7 +301,7 @@ sub Motion
  if (defined($new) && $new->IsMenubutton && $new->cget('-indicatoron') == 0 &&
      $w->cget('-indicatoron') == 0)
   {
-   if ($upDown eq "down")
+   if ($upDown eq 'down')
     {
      $new->Post($rootx,$rooty);
     }
@@ -322,18 +322,17 @@ sub Motion
 sub ButtonUp {
     my $w = shift;
 
-    my $menu = $Tk::postedMb->cget(-menu);
-    my $tearoff = $Tk::platform eq 'unix' || (defined($menu) && 
-					      $menu->cget('-type') eq 'tearoff');
+    my $tearoff = $Tk::platform eq 'unix' || (defined($menu) &&
+					      $w->cget(-menu)->cget('-type') eq 'tearoff');
     if (($tearoff != 0) && (defined($Tk::postedMb) && $Tk::postedMb == $w)
 	    && (defined($Tk::inMenubutton) && $Tk::inMenubutton == $w)) {
-	$menu->FirstEntry();
+	$Tk::postedMb->cget(-menu)->FirstEntry();
     } else {
       Tk::Menu->Unpost(undef);
     }
 } # end ButtonUp
 
-# Some convenience methods 
+# Some convenience methods
 
 sub menu
 {
@@ -342,7 +341,7 @@ sub menu
  if (!defined $menu)
   {
    require Tk::Menu;
-   $w->ColorOptions(\%args) if ($Tk::platform eq 'unix'); 
+   $w->ColorOptions(\%args) if ($Tk::platform eq 'unix');
    $menu = $w->Menu(%args);
    $w->configure('-menu'=>$menu);
   }
@@ -378,13 +377,14 @@ sub FindMenu
 {
  my $child = shift;
  my $char = shift;
- my $ul = $child->cget("-underline");
- if (defined $ul && $ul >= 0 && $child->cget("-state") ne "disabled")
+ my $ul = $child->cget('-underline');
+ if (defined $ul && $ul >= 0 && $child->cget('-state') ne 'disabled')
   {
-   my $char2 = $child->cget("-text");
+   my $char2 = $child->cget('-text');
    $char2 = substr("\L$char2",$ul,1) if (defined $char2);
-   if (!defined($char) || $char eq "" || (defined($char2) && "\l$char" eq $char2))
+   if (!defined($char) || $char eq '' || (defined($char2) && "\l$char" eq $char2))
     {
+     $child->PostFirst;
      return $child;
     }
   }
