@@ -1,7 +1,7 @@
 package Tk::Dialog;
 
 use vars qw($VERSION);
-$VERSION = '3.013'; # $Id: //depot/Tk8/Tk/Dialog.pm#13$
+$VERSION = '3.024'; # $Id: //depot/Tk8/Tk/Dialog.pm#24$
 
 # Dialog - a translation of `tk_dialog' from Tcl/Tk to TkPerl (based on
 # John Stoffel's idea).
@@ -9,20 +9,24 @@ $VERSION = '3.013'; # $Id: //depot/Tk8/Tk/Dialog.pm#13$
 # Stephen O. Lidie, Lehigh University Computing Center.  94/12/27
 # lusol@Lehigh.EDU
 
+# Documentation after __END__
+
 use Carp;
 use strict;
 
 require Tk::Toplevel;
-@Tk::Dialog::ISA = qw(Tk::Toplevel);
+use base qw(Tk::Toplevel);
 
 Construct Tk::Widget 'Dialog';
 
-sub Populate {
+sub Populate
+{
 
     # Dialog object constructor.  Uses `new' method from base class
     # to create object container then creates the dialog toplevel.
 
     my($cw, $args) = @_;
+
     $cw->SUPER::Populate($args);
 
     my ($w_bitmap,$w_but,$pad1,$pad2);
@@ -31,15 +35,17 @@ sub Populate {
     $buttons = ['OK'] unless (defined $buttons);
     my $default_button = delete $args->{-default_button};
     $default_button =  $buttons->[0] unless (defined $default_button);
-    $cw->{-default_button_text} = $default_button;
+
 
     # Create the Toplevel window and divide it into top and bottom parts.
 
     $cw->{'selected_button'} = '';
     my (@pl) = (-side => 'top', -fill => 'both');
+
     ($pad1, $pad2) =
         ([-padx => '3m', -pady => '3m'], [-padx => '3m', -pady => '2m']);
 
+
     $cw->iconname('Dialog');
     $cw->protocol('WM_DELETE_WINDOW' => sub {});
     $cw->transient($cw->Parent->toplevel);
@@ -49,7 +55,6 @@ sub Populate {
     my $w_bot = $cw->Frame(Name => 'bot',-relief => 'raised', -borderwidth => 1);
     $w_top->pack(@pl);
     $w_bot->pack(@pl);
-    $cw->Advertise(-buttons_frame => $w_bot);
 
     # Fill the top part with the bitmap and message.
 
@@ -62,11 +67,49 @@ sub Populate {
 
     $w_msg->pack(-side => 'right', -expand => 1, -fill => 'both', @$pad1);
 
+    # Create a row of buttons at the bottom of the dialog.
+
+    my($w_default_button, $bl) = (undef, '');
+    foreach $bl (@$buttons) {
+        $w_but = $w_bot->Button(
+            -text => $bl,
+            -command => [
+                sub {
+                    $_[0]->{'selected_button'} = $_[1];
+                }, $cw, $bl,
+            ]
+        );
+        if ($Tk::platform eq 'MSWin32') {
+            $w_but->configure(-width => 10, -pady => 0);
+        }
+
+        if ($bl eq $default_button) {
+            if ($Tk::platform eq 'MSWin32') {
+                $w_default_button = $w_but;
+                $w_but->configure(-borderwidth => 2);
+                $w_but->pack(@pl, -expand => 1, @$pad2);
+            } else {
+                my $w_default_frame = $w_bot->Frame(
+                    -relief      => 'sunken',
+                    -borderwidth => 1
+                );
+                $w_but->raise($w_default_frame);
+                $w_default_frame->pack(@pl, -expand => 1, @$pad2);
+                $w_but->pack(-in => $w_default_frame, -padx => '2m',
+                             -pady => '2m');
+            }
+            $cw->bind( '<Return>' => ['Return', $w_but, $cw, $bl ]);
+            $w_default_button = $w_but;
+        } else {
+         $w_but->pack(@pl, -expand => 1, @$pad2);
+        }
+    } # forend all buttons
+
     $cw->Advertise(message => $w_msg);
     $cw->Advertise(bitmap  => $w_bitmap );
+    $cw->{'default_button'} = $w_default_button;
 
     $cw->ConfigSpecs(
-                      -buttons    => ['METHOD', 'buttons', 'Buttons', ['OK']],
                       -image      => ['bitmap',undef,undef,undef],
                       -bitmap     => ['bitmap',undef,undef,undef],
                       -fg         => ['ADVERTISED','foreground','Foreground','black'],
@@ -77,53 +120,13 @@ sub Populate {
                       DEFAULT     => ['message',undef,undef,undef]
                      );
     $cw->Delegates('Construct' => $w_top);
-
 } # end Dialog constructor
 
-sub buttons {
+sub Return {
+ $_[1]->flash;
+ $_[2]->{'selected_button'} = $_[3];
+}
 
-    # Create a row of buttons at the bottom of the dialog.
-
-    my($cw, $buttons) = @_;
-
-    my($w_default_button, $bl) = (undef, '');
-    my $w_bot = $cw->Subwidget(-buttons_frame);
-    my (@pl) = (-side => 'left');
-    my $pad2 = [-padx => '3m', -pady => '2m'];
-    my($w_but);
-    foreach $bl (@$buttons) {
-        $w_but = $w_bot->Button(
-            -text => $bl,
-            -command => [
-                sub {
-                    $_[0]->{'selected_button'} = $_[1];
-                }, $cw, $bl,
-            ]
-        );
-        if ($bl eq $cw->{-default_button_text}) {
-            $w_default_button = $w_bot->Frame(
-                -relief      => 'sunken',
-                -borderwidth => 1
-            );
-            $w_but->raise($w_default_button);
-            $w_default_button->pack(@pl, -expand => 1, @$pad2);
-            $w_but->pack(-in => $w_default_button, -padx => '2m',
-                         -pady => '2m');
-            $cw->bind(
-                '<Return>' => [
-                    sub {
-                        $_[1]->flash; 
-                        $_[2]->{'selected_button'} = $_[3];
-                    }, $w_but, $cw, $bl,
-                ]
-            );
-        } else {
-         $w_but->pack(@pl, -expand => 1, @$pad2);
-        }
-    } # forend all buttons
-    $cw->{'default_button'} = $w_default_button;
-
-} # end buttons
 
 sub Show {
 
@@ -131,7 +134,7 @@ sub Show {
 
     my ($cw, $grab_type) = @_;
 
-    croak "Dialog:  `Show' method requires at least 1 argument"
+    croak 'Dialog:  "Show" method requires at least 1 argument'
         if scalar @_ < 1 ;
 
     my $old_focus = $cw->focusSave;
@@ -140,7 +143,7 @@ sub Show {
     # Update all geometry information, center the dialog in the display
     # and deiconify it
 
-    $cw->Popup(); 
+    $cw->Popup();
 
     # set a grab and claim the focus.
 
@@ -151,11 +154,11 @@ sub Show {
     }
     $cw->waitVisibility;
     $cw->update;
-    if (defined $cw->{'default_button'}) 
+    if (defined $cw->{'default_button'})
      {
       $cw->{'default_button'}->focus;
-     } 
-    else 
+     }
+    else
      {
       $cw->focus;
      }
@@ -173,3 +176,8 @@ sub Show {
 } # end Dialog Show method
 
 1;
+
+__END__
+
+=cut
+
