@@ -12,15 +12,15 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 
-package Tk::Menu; 
+package Tk::Menu;
 require Tk;
 require Tk::Widget;
 require Tk::Wm;
 require Tk::Derived;
 
 
-use vars qw($VERSION @ISA);
-$VERSION = '3.020'; # $Id: //depot/Tk8/Tk/Menu.pm#20$
+use vars qw($VERSION);
+$VERSION = '3.030'; # $Id: //depot/Tk8/Tk/Menu.pm#30$
 
 use strict;
 
@@ -28,11 +28,11 @@ use base  qw(Tk::Wm Tk::Derived Tk::Widget);
 
 Construct Tk::Widget 'Menu';
 
-sub Tk_cmd { \&Tk::menu }    
+sub Tk_cmd { \&Tk::menu }
 
-Tk::Methods("activate","add","clone","delete","entrycget","entryconfigure",
-            "index","insert","invoke","post","postcascade","type",
-            "unpost","yposition");
+Tk::Methods('activate','add','clone','delete','entrycget','entryconfigure',
+            'index','insert','invoke','post','postcascade','type',
+            'unpost','yposition');
 
 import Tk qw(Ev);
 
@@ -46,7 +46,7 @@ sub CreateArgs
  my $opt;
  foreach $opt (qw(-type -screen -visual -colormap))
   {
-   my $val = delete $args->{$opt};                     
+   my $val = delete $args->{$opt};
    push(@result, $opt => $val) if (defined $val);
   }
  return @result;
@@ -65,7 +65,7 @@ sub InitObject
      $menu->configure(%$args);
      %$args = ();
     }
-   $menu->AddItems(@$menuitems) 
+   $menu->AddItems(@$menuitems)
   }
 }
 
@@ -78,9 +78,9 @@ sub AddItems
   {
    my $item = shift;
    if (!ref($item))
-    { 
+    {
      $menu->separator;  # A separator
-    }  
+    }
    else
     {
      my ($kind,$name,%minfo) = ( @$item );
@@ -97,7 +97,7 @@ sub AddItems
     }  # A non-separator
   }
 }
-        
+
 #
 #-------------------------------------------------------------------------
 # Elements of tkPriv that are used in this file:
@@ -157,22 +157,23 @@ sub ClassInit
  my ($class,$mw) = @_;
  # Must set focus when mouse enters a menu, in order to allow
  # mixed-mode processing using both the mouse and the keyboard.
- $mw->bind($class,"<FocusIn>", 'NoOp');
- $mw->bind($class,"<Enter>", 'Enter');
- $mw->bind($class,"<Leave>", ['Leave',Ev('X'),Ev('Y'),Ev('s')]);
- $mw->bind($class,"<Motion>", ['Motion',Ev('x'),Ev('y'),Ev('s')]);
- $mw->bind($class,"<ButtonPress>",'ButtonDown');
- $mw->bind($class,"<ButtonRelease>",['Invoke',1]);
- $mw->bind($class,"<space>",['Invoke',0]);
- $mw->bind($class,"<Return>",['Invoke',0]);
- $mw->bind($class,"<Escape>",'Escape');
- $mw->bind($class,"<Left>",'LeftArrow');
- $mw->bind($class,"<Right>",'RightArrow');
- $mw->bind($class,"<Up>",'UpArrow');
- $mw->bind($class,"<Down>",'DownArrow');
- $mw->bind($class,"<KeyPress>", ['TraverseWithinMenu',Ev('A')]);
+ $mw->bind($class,'<FocusIn>', 'NoOp');
+ $mw->bind($class,'<Enter>', 'Enter');
+ $mw->bind($class,'<Leave>', ['Leave',Ev('X'),Ev('Y'),Ev('s')]);
+ $mw->bind($class,'<Motion>', ['Motion',Ev('x'),Ev('y'),Ev('s')]);
+ $mw->bind($class,'<ButtonPress>','ButtonDown');
+ $mw->bind($class,'<ButtonRelease>',['Invoke',1]);
+ $mw->bind($class,'<space>',['Invoke',0]);
+ $mw->bind($class,'<Return>',['Invoke',0]);
+ $mw->bind($class,'<Escape>','Escape');
+ $mw->bind($class,'<Left>','LeftArrow');
+ $mw->bind($class,'<Right>','RightArrow');
+ $mw->bind($class,'<Up>','UpArrow');
+ $mw->bind($class,'<Down>','DownArrow');
+ $mw->bind($class,'<KeyPress>', ['TraverseWithinMenu',Ev('K')]);
+ $mw->bind($class,'<Alt-KeyPress>', ['TraverseWithinMenu',Ev('K')]);
  return $class;
-}                     
+}
 
 sub UpArrow
 {
@@ -198,7 +199,7 @@ sub DownArrow
   {
    $menu->NextEntry(1);
   }
-}                   
+}
 
 sub LeftArrow
 {
@@ -257,23 +258,26 @@ sub Unpost
 
  # Unpost menu(s) and restore some stuff that's dependent on
  # what was posted.
- eval {local $SIG{__DIE__}; 
+ eval {local $SIG{__DIE__};
    if (defined $mb)
      {
-      $menu = $mb->cget("-menu");
+      $menu = $mb->cget('-menu');
       $menu->unpost();
       $Tk::postedMb = undef;
-      $mb->configure("-cursor",$Tk::cursor);
-      $mb->configure("-relief",$Tk::relief)
+      $mb->configure('-cursor',$Tk::cursor);
+      $mb->configure('-relief',$Tk::relief)
      }
     elsif (defined $Tk::popup)
      {
       $Tk::popup->unpost();
+      my $grab = $Tk::popup->grabCurrent;
+      $grab->grabRelease if (defined $grab);
+
       undef $Tk::popup;
      }
-    elsif (defined $menu && ref $menu && 
+    elsif (defined $menu && ref $menu &&
            $menu->cget('-type') ne 'menubar' &&
-           $menu->cget('-type') ne 'tearoff' 
+           $menu->cget('-type') ne 'tearoff'
           )
      {
       # We're in a cascaded sub-menu from a torn-off menu or popup.
@@ -284,8 +288,9 @@ sub Unpost
        {
         my $parent = $menu->parent;
         last if (!$parent->IsMenu || !$parent->ismapped);
-        $parent->postcascade("none");        
+        $parent->postcascade('none');
         $parent->GenerateMenuSelect;
+        $parent->activate('none');
         my $type = $parent->cget('-type');
         last if ($type eq 'menubar' || $type eq 'tearoff');
         $menu = $parent
@@ -313,13 +318,13 @@ sub Unpost
      undef $Tk::tearoff;
     }
   }
-}   
+}
 
 sub RestoreOldGrab
 {
  if (defined $Tk::oldGrab)
   {
-   eval 
+   eval
     {
      local $SIG{__DIE__};
      if ($Tk::grabStatus eq 'global')
@@ -330,7 +335,7 @@ sub RestoreOldGrab
       {
        $Tk::oldGrab->grab;
       }
-    };     
+    };
    undef $Tk::oldGrab;
   }
 }
@@ -361,7 +366,7 @@ sub Motion
  my $t     = $menu->cget('-type');
 
  if ($menu->IS($Tk::window))
-  {                     
+  {
    if ($menu->cget('-type') eq 'menubar')
     {
 #    if (defined($Tk::focus) && $Tk::focus != $menu)
@@ -378,7 +383,7 @@ sub Motion
   }
  if (($state & 0x1f00) != 0)
   {
-   $menu->postcascade("active")
+   $menu->postcascade('active')
   }
 }
 # ButtonDown --
@@ -398,7 +403,7 @@ sub Motion
 sub ButtonDown
 {
  my $menu = shift;
- $menu->postcascade("active");
+ $menu->postcascade('active');
  if (defined $Tk::postedMb)
   {
    $Tk::postedMb->grabGlobal
@@ -407,12 +412,12 @@ sub ButtonDown
   {
    while ($menu->cget('-type') eq 'normal'
           && $menu->parent->IsMenu
-          && $menu->parent->ismapped 
+          && $menu->parent->ismapped
          )
     {
      $menu = $menu->parent;
     }
-   
+
    if (!defined $Tk::menuBar)
     {
      $Tk::menuBar = $menu;
@@ -423,7 +428,7 @@ sub ButtonDown
    # Don't update grab information if the grab window isn't changing.
    # Otherwise, we'll get an error when we unpost the menus and
    # restore the grab, since the old grab window will not be viewable
-   # anymore.              
+   # anymore.
 
    $menu->SaveGrabInfo unless ($menu->IS($menu->grabCurrent));
 
@@ -432,13 +437,13 @@ sub ButtonDown
 
    $menu->grabGlobal if ($Tk::platform eq 'unix');
   }
-}            
+}
 
 sub Enter
 {
- my $w = shift; 
+ my $w = shift;
  my $ev = $w->XEvent;
- $Tk::window = $w; 
+ $Tk::window = $w;
  if ($w->cget('-type') eq 'tearoff')
   {
    if ($ev->m ne 'NotifyUngrab')
@@ -465,13 +470,13 @@ sub Leave
  my $rooty = shift;
  my $state = shift;
  undef $Tk::window;
- return if ($menu->index("active") eq "none");
- if ($menu->typeIS("active","cascade"))
+ return if ($menu->index('active') eq 'none');
+ if ($menu->typeIS('active','cascade'))
   {
    my $c = $menu->Containing($rootx,$rooty);
-   return if (defined $c && $menu->entrycget("active","-menu")->IS($c));
+   return if (defined $c && $menu->entrycget('active','-menu')->IS($c));
   }
- $menu->activate("none");
+ $menu->activate('none');
  $menu->GenerateMenuSelect;
 }
 
@@ -500,19 +505,19 @@ sub Invoke
    return;
   }
 
- my $type = $w->type("active");
- if ($w->typeIS("active","cascade"))
+ my $type = $w->type('active');
+ if ($w->typeIS('active','cascade'))
   {
-   $w->postcascade("active");
-   my $menu = $w->entrycget("active","-menu");
+   $w->postcascade('active');
+   my $menu = $w->entrycget('active','-menu');
    $menu->FirstEntry() if (defined $menu);
   }
- elsif ($w->typeIS("active","tearoff"))
+ elsif ($w->typeIS('active','tearoff'))
   {
    $w->Unpost();
    $w->TearOffMenu();
   }
- elsif ($w->typeIS("active","menubar"))
+ elsif ($w->typeIS('active','menubar'))
   {
    $w->postcascade('none');
    $w->activate('none');
@@ -522,7 +527,7 @@ sub Invoke
  else
   {
    $w->Unpost();
-   $w->invoke("active")
+   $w->invoke('active')
   }
 }
 # Escape --
@@ -534,7 +539,7 @@ sub Invoke
 # menu - Name of the menu window.
 sub Escape
 {
- my $menu = shift;  
+ my $menu = shift;
  my $parent = $menu->parent;
  if (!$parent->IsMenu)
   {
@@ -565,13 +570,13 @@ sub NextMenu
  my $direction = shift;
  # First handle traversals into and out of cascaded menus.
  my $count;
- if ($direction eq "right")
+ if ($direction eq 'right')
   {
    $count = 1;
-   if ($menu->typeIS("active","cascade"))
+   if ($menu->typeIS('active','cascade'))
     {
-     $menu->postcascade("active");
-     my $m2 = $menu->entrycget("active","-menu");
+     $menu->postcascade('active');
+     my $m2 = $menu->entrycget('active','-menu');
      $m2->FirstEntry if (defined $m2);
      return;
     }
@@ -586,7 +591,7 @@ sub NextMenu
          $parent->NextEntry(1);
          return;
         }
-       $parent = $parent->parent; 
+       $parent = $parent->parent;
       }
     }
   }
@@ -595,7 +600,7 @@ sub NextMenu
    $count = -1;
    my $m2 = $menu->parent;
    if ($m2->IsMenu)
-    { 
+    {
      if ($m2->cget('-type') ne 'menubar')
       {
        $menu->activate('none');
@@ -603,8 +608,8 @@ sub NextMenu
        $m2->SetFocus;
        # This code unposts any posted submenu in the parent.
        my $tmp = $m2->index('active');
-       $m2->activate('none'); 
-       $m2->activate($tmp); 
+       $m2->activate('none');
+       $m2->activate($tmp);
        return;
       }
     }
@@ -640,7 +645,7 @@ sub NextMenu
      $i += -$length
     }
    $mb = $buttons[$i];
-   last if ($mb->IsMenubutton && $mb->cget("-state") ne "disabled"
+   last if ($mb->IsMenubutton && $mb->cget('-state') ne 'disabled'
             && defined($mb->cget('-menu'))
             && $mb->cget('-menu')->index('last') ne 'none'
            );
@@ -661,14 +666,14 @@ sub NextEntry
 {
  my $menu = shift;
  my $count = shift;
- if ($menu->index("last") eq "none")
+ if ($menu->index('last') eq 'none')
   {
    return;
   }
- my $length = $menu->index("last")+1;
+ my $length = $menu->index('last')+1;
  my $quitAfter = $length;
- my $active = $menu->index("active");
- my $i = ($active eq "none") ? 0 : $active+$count; 
+ my $active = $menu->index('active');
+ my $i = ($active eq 'none') ? 0 : $active+$count;
  while (1)
   {
    return if ($quitAfter <= 0);
@@ -680,13 +685,13 @@ sub NextEntry
     {
      $i += -$length
     }
-   my $state = eval {local $SIG{__DIE__};  $menu->entrycget($i,"-state") };
-   last if (defined($state) && $state ne "disabled");
+   my $state = eval {local $SIG{__DIE__};  $menu->entrycget($i,'-state') };
+   last if (defined($state) && $state ne 'disabled');
    return if ($i == $active);
    $i += $count;
    $quitAfter -= 1;
   }
- $menu->activate($i); 
+ $menu->activate($i);
  $menu->GenerateMenuSelect;
  if ($menu->type($i) eq 'cascade')
   {
@@ -713,13 +718,13 @@ sub TraverseWithinMenu
  my $char = shift;
  return unless (defined $char);
  $char = "\L$char";
- my $last = $w->index("last");
- return if ($last eq "none");
+ my $last = $w->index('last');
+ return if ($last eq 'none');
  for (my $i = 0;$i <= $last;$i += 1)
   {
-   my $label = eval {local $SIG{__DIE__};  $w->entrycget($i,"-label") };
+   my $label = eval {local $SIG{__DIE__};  $w->entrycget($i,'-label') };
    next unless defined($label);
-   my $ul = $w->entrycget($i,"-underline");
+   my $ul = $w->entrycget($i,'-underline');
    if (defined $ul && $ul >= 0)
     {
      $label = substr("\L$label",$ul,1);
@@ -734,7 +739,7 @@ sub TraverseWithinMenu
         }
        else
         {
-         $w->Unpost();  
+         $w->Unpost();
          $w->invoke($i);
         }
        return;
@@ -742,6 +747,26 @@ sub TraverseWithinMenu
     }
   }
 }
+
+sub FindMenu
+{
+ my ($menu,$char) = @_;
+ if ($menu->cget('-type') eq 'menubar')
+  {
+   if (!defined($char) || $char eq '')
+    {
+     $menu->FirstEntry;
+    }
+   else
+    {
+     $menu->TraverseWithinMenu($char);
+    }
+   return $menu;
+  }
+ return undef;
+}
+
+
 # FirstEntry --
 # Given a menu, this procedure finds the first entry that isn't
 # disabled or a tear-off or separator, and activates that entry.
@@ -755,15 +780,15 @@ sub TraverseWithinMenu
 sub FirstEntry
 {
  my $menu = shift;
- return if (!defined($menu) || $menu eq "" || !ref($menu));
+ return if (!defined($menu) || $menu eq '' || !ref($menu));
  $menu->SetFocus;
- return if ($menu->index("active") ne "none");
- my $last = $menu->index("last");
+ return if ($menu->index('active') ne 'none');
+ my $last = $menu->index('last');
  return if ($last eq 'none');
  for (my $i = 0;$i <= $last;$i += 1)
   {
-   my $state = eval {local $SIG{__DIE__};  $menu->entrycget($i,"-state") };
-   if (defined $state && $state ne "disabled" && !$menu->typeIS($i,"tearoff"))
+   my $state = eval {local $SIG{__DIE__};  $menu->entrycget($i,'-state') };
+   if (defined $state && $state ne 'disabled' && !$menu->typeIS($i,'tearoff'))
     {
      $menu->activate($i);
      $menu->GenerateMenuSelect;
@@ -801,11 +826,11 @@ sub FindName
    $i = eval {local $SIG{__DIE__};  $menu->index($s) };
    return $i;
   }
- my $last = $menu->index("last");
+ my $last = $menu->index('last');
  return if ($last eq 'none');
  for ($i = 0;$i <= $last;$i += 1)
   {
-   my $label = eval {local $SIG{__DIE__};  $menu->entrycget($i,"-label") };
+   my $label = eval {local $SIG{__DIE__};  $menu->entrycget($i,'-label') };
    return $i if (defined $label && $label eq $s);
   }
  return undef;
@@ -829,7 +854,7 @@ sub PostOverPoint
  my $entry = shift;
  if (defined $entry)
   {
-   if ($entry == $menu->index("last"))
+   if ($entry == $menu->index('last'))
     {
      $y -= ($menu->yposition($entry)+$menu->height)/2;
     }
@@ -840,7 +865,7 @@ sub PostOverPoint
    $x -= $menu->reqwidth/2;
   }
  $menu->post($x,$y);
- if (defined($entry) && $menu->entrycget($entry,"-state") ne "disabled")
+ if (defined($entry) && $menu->entrycget($entry,'-state') ne 'disabled')
   {
    $menu->activate($entry);
    $menu->GenerateMenuSelect;
@@ -873,7 +898,7 @@ sub Post
 }
 
 sub SetFocus
-{                 
+{
  my $menu = shift;
  $Tk::focus = $menu->focusCurrent if (!defined($Tk::focus));
  $menu->focus;
@@ -884,7 +909,7 @@ sub GenerateMenuSelect
  my $menu = shift;
  $Tk::activeMenu = $menu;
  $Tk::activeItem = $menu->index('active');
- $menu->eventGenerate('<<MenuSelect>>');  # FIXME 
+ $menu->eventGenerate('<<MenuSelect>>');  # FIXME
 }
 
 # Converted from tearoff.tcl --
@@ -940,11 +965,11 @@ sub TearOffMenu
    $parent = $w->parent;
    if ($parent->IsMenubutton)
     {
-     $title = $parent->cget("-text");
+     $title = $parent->cget('-text');
     }
    elsif ($parent->IsMenu)
     {
-     $title = $parent->entrycget("active","-label");
+     $title = $parent->entrycget('active','-label');
     }
   }
  $menu->title($title) if (defined $title && length($title));
@@ -952,7 +977,7 @@ sub TearOffMenu
  # Set tkPriv(focus) on entry: otherwise the focus will get lost
  # after keyboard invocation of a sub-menu (it will stay on the
  # submenu).
- $menu->bind("<Enter>",'EnterFocus');
+ $menu->bind('<Enter>','EnterFocus');
  $menu->Callback('-tearoffcommand');
  return $menu;
 }
@@ -982,10 +1007,10 @@ sub MenuDup
   {
    $dst->transient($parent->MainWindow);
   }
- my $last = $src->index("last");
+ my $last = $src->index('last');
  if ($last ne 'none')
   {
-   for (my $i = $src->cget("-tearoff"); $i <= $last; $i++)
+   for (my $i = $src->cget('-tearoff'); $i <= $last; $i++)
     {
      my $type = $src->type($i);
      if (defined $type)
@@ -996,16 +1021,16 @@ sub MenuDup
          next if (@$option == 2);
          push(@args,$$option[0],$$option[4]) if (defined $$option[4]);
         }
-       $dst->add($type,@args);  
-      }           
-     
+       $dst->add($type,@args);
+      }
+
      # Duplicate the binding tags and bindings from the source menu.
      my @bindtags = $src->bindtags;
      my $path = $src->PathName;
      foreach (@bindtags)
       {
        $_ = $dst if ($_ eq $path);
-      }                            
+      }
      $dst->bindtags([@bindtags]);
      foreach my $event ($src->bind)
       {
@@ -1015,11 +1040,11 @@ sub MenuDup
     }
   }
  return $dst;
-} 
+}
 
 
 
-# Some convenience methods 
+# Some convenience methods
 
 sub separator   { require Tk::Menu::Item; shift->Separator(@_);   }
 sub command     { require Tk::Menu::Item; shift->Command(@_);     }
