@@ -6,7 +6,7 @@ use Carp;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '3.012'; # $Id: //depot/Tk8/Tk/Menu/Item.pm#12$
+$VERSION = '3.014'; # $Id: //depot/Tk8/Tk/Menu/Item.pm#14$
 
 sub PreInit
 {
@@ -40,11 +40,11 @@ sub new
    else
     {
      $name = $minfo{'-bitmap'} || $minfo{'-image'};
-     croak("No -label") unless defined($name);
+     croak('No -label') unless defined($name);
      $minfo{'-label'} = $name;
     }
-   $class->PreInit($menu,\%minfo);    
-   $menu->add($kind,%minfo);          
+   $class->PreInit($menu,\%minfo);
+   $menu->add($kind,%minfo);
    $menu->invoke('last') if ($invoke);
   }
  else
@@ -52,7 +52,7 @@ sub new
    $menu->add('separator');
   }
  return bless [$menu,$name],$class;
-} 
+}
 
 sub configure
 {
@@ -77,23 +77,23 @@ sub parentMenu
 # Default "kind" is a command
 sub kind { return 'command' }
 
-# Now the derived packages 
+# Now the derived packages
 
 package Tk::Menu::Separator;
-@Tk::Menu::Separator::ISA = qw(Tk::Menu::Item);
+use base qw(Tk::Menu::Item);
 Construct Tk::Menu 'Separator';
 sub kind { return undef }
 
 package Tk::Menu::Button;
-@Tk::Menu::Button::ISA = qw(Tk::Menu::Item);
+use base qw(Tk::Menu::Item);
 Construct Tk::Menu 'Button';
 
 package Tk::Menu::Command;
-@Tk::Menu::Command::ISA = qw(Tk::Menu::Button);
+use base qw(Tk::Menu::Button);
 Construct Tk::Menu 'Command';
 
 package Tk::Menu::Cascade;
-@Tk::Menu::Cascade::ISA = qw(Tk::Menu::Item);
+use base qw(Tk::Menu::Item);
 Construct Tk::Menu 'Cascade';
 sub kind { return 'cascade' }
 use Carp;
@@ -122,7 +122,7 @@ sub menu
  if (!defined $menu)
   {
    require Tk::Menu;
-   $w->ColorOptions(\%args); 
+   $w->ColorOptions(\%args);
    my $name = $self->cget('-label');
    warn "Had to (re-)reate menu for $name";
    $menu = $w->Menu(Name => $name, %args);
@@ -133,9 +133,9 @@ sub menu
    $menu->configure(%args);
   }
  return $menu;
-}             
+}
 
-# Some convenience methods 
+# Some convenience methods
 
 sub separator   {  shift->menu->Separator(@_);   }
 sub command     {  shift->menu->Command(@_);     }
@@ -143,19 +143,19 @@ sub cascade     {  shift->menu->Cascade(@_);     }
 sub checkbutton {  shift->menu->Checkbutton(@_); }
 sub radiobutton {  shift->menu->Radiobutton(@_); }
 
-sub pack 
-{                    
+sub pack
+{
  my $w = shift;
  carp "Cannot 'pack' $w - done automatically" if $^W;
 }
 
 package Tk::Menu::Checkbutton;
-@Tk::Menu::Checkbutton::ISA = qw(Tk::Menu::Item);
+use base qw(Tk::Menu::Item);
 Construct Tk::Menu 'Checkbutton';
 sub kind { return 'checkbutton' }
 
 package Tk::Menu::Radiobutton;
-@Tk::Menu::Radiobutton::ISA = qw(Tk::Menu::Item);
+use base qw(Tk::Menu::Item);
 Construct Tk::Menu 'Radiobutton';
 sub kind { return 'radiobutton' }
 
