@@ -6,17 +6,21 @@
  *
  * Copyright (c) 1990-1994 The Regents of the University of California.
  * Copyright (c) 1994-1995 Sun Microsystems, Inc.
+ * Copyright (c) 1998 by Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkEvent.c 1.20 96/09/20 09:33:38
+ * RCS: @(#) $Id: tkEvent.c,v 1.3 1998/10/10 00:30:36 rjohnson Exp $
  */
 
 #include "tkPort.h"
 #include "tkInt.h"
 #include <signal.h>
 
+/* This does not belong here it belong in the platform window manager code! */
+extern Window XmuClientWindow _ANSI_ARGS_((Display *dpy, Window win));
+
 /*
  * There's a potential problem if a handler is deleted while it's
  * current (i.e. its procedure is executing), since Tk_HandleEvent
@@ -129,7 +133,8 @@ static unsigned long eventMasks[TK_LASTEVENT] = {
     0,					/* Mapping Notify */
     VirtualEventMask,			/* VirtualEvents */
     ActivateMask,			/* ActivateNotify */
-    ActivateMask			/* DeactivateNotify */
+    ActivateMask,			/* DeactivateNotify */
+    MouseWheelMask			/* MouseWheelEvent */
 };
 
 /*
@@ -491,6 +496,16 @@ Tk_HandleEvent(eventPtr)
 	}
     }
     winPtr = (TkWindow *) Tk_IdToWindow(eventPtr->xany.display, handlerWindow);
+                        
+    /* Some Drag&Drop messages get sent to wrappers for which there is
+     * an X ICCM scheme to locate the "real" window - it is not clear 
+     * if this is "done right" yet but should be harmless.
+     */
+    if (winPtr == NULL && eventPtr->type == ClientMessage) {
+        handlerWindow = XmuClientWindow(eventPtr->xany.display, handlerWindow);
+	winPtr = (TkWindow *) Tk_IdToWindow(eventPtr->xany.display, handlerWindow);
+    }
+
     if (winPtr == NULL) {
 
 	/*
@@ -537,7 +552,7 @@ Tk_HandleEvent(eventPtr)
 	 * and Leave events;  depending on its return value, ignore the
 	 * event.
 	 */
-    
+
 	if ((mask & (FocusChangeMask|EnterWindowMask|LeaveWindowMask))
 		&& !TkFocusFilterEvent(winPtr, eventPtr)) {
             Tcl_Release((ClientData) interp);
@@ -546,10 +561,13 @@ Tk_HandleEvent(eventPtr)
     
 	/*
 	 * Redirect KeyPress and KeyRelease events to the focus window,
-	 * or ignore them entirely if there is no focus window.
+	 * or ignore them entirely if there is no focus window.  We also
+	 * route the MouseWheel event to the focus window.  The MouseWheel
+	 * event is an extension to the X event set.  Currently, it is only
+	 * available on the Windows version of Tk.
 	 */
     
-	if (mask & (KeyPressMask|KeyReleaseMask)) {
+	if (mask & (KeyPressMask|KeyReleaseMask|MouseWheelMask)) {
 	    winPtr->dispPtr->lastEventTime = eventPtr->xkey.time;
 	    winPtr = TkFocusKeyEvent(winPtr, eventPtr);
 	    if (winPtr == NULL) {
