@@ -8,11 +8,12 @@
  *	tcl*Notify.c files in each platform directory.
  *
  * Copyright (c) 1995-1997 Sun Microsystems, Inc.
+ * Copyright (c) 1998 by Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclNotify.c 1.16 97/09/15 15:12:52
+ * RCS: @(#) $Id: tclNotify.c,v 1.3 1998/10/14 21:12:09 rjohnson Exp $
  */
 
 #include "tkPort.h"
@@ -344,9 +345,12 @@ Tcl_DeleteEvents(proc, clientData)
         if ((*proc) (evPtr, clientData) == 1) {
             if (notifier.firstEventPtr == evPtr) {
                 notifier.firstEventPtr = evPtr->nextPtr;
-                if (evPtr->nextPtr == (Tcl_Event *) NULL) {
-                    notifier.lastEventPtr = (Tcl_Event *) NULL;
+                if (evPtr->nextPtr == NULL) {
+                    notifier.lastEventPtr = prevPtr;
                 }
+		if (notifier.markerEventPtr == evPtr) {
+		    notifier.markerEventPtr = prevPtr;
+		}
             } else {
                 prevPtr->nextPtr = evPtr->nextPtr;
             }
