@@ -10,16 +10,12 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclWinNotify.c 1.17 97/05/23 10:48:44
+ * RCS: @(#) $Id: tclWinNotify.c,v 1.2 1998/09/14 18:40:20 stanton Exp $
  */
 
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
-#ifdef __GNUC__
-#include <Windows32/Sockets.h>
-#else
 #include <winsock.h>
-#endif
 
 /*
  * The follwing static indicates whether this module has been initialized.
